@@ -27,38 +27,37 @@ namespace TagLib.Id3v2
 {
    public class UniqueFileIdentifierFrame : Frame
    {
-      //////////////////////////////////////////////////////////////////////////
-      // private properties
-      //////////////////////////////////////////////////////////////////////////
-      private string owner;
-      private ByteVector identifier;
+      #region Private Properties
+      private string owner = null;
+      private ByteVector identifier = null;
+      #endregion
       
       
-      //////////////////////////////////////////////////////////////////////////
-      // public methods
-      //////////////////////////////////////////////////////////////////////////
-      public UniqueFileIdentifierFrame (ByteVector data) : base (data)
+      
+      #region Constructors
+      public UniqueFileIdentifierFrame (ByteVector data, byte version) : base (data, version)
       {
-         owner = null;
-         identifier = null;
-         
-         SetData (data, 0);
+         SetData (data, 0, version, true);
       }
 
-      public UniqueFileIdentifierFrame (string owner, ByteVector id) : base ("UFID")
+      public UniqueFileIdentifierFrame (string owner, ByteVector id) : base (FrameType.UFID, 4)
       {
          this.owner = owner;
          identifier = id;
       }
       
+      protected internal UniqueFileIdentifierFrame (ByteVector data, int offset, FrameHeader header, byte version) : base(header)
+      {
+         SetData (data, offset, version, false);
+      }
+      #endregion
       
-      //////////////////////////////////////////////////////////////////////////
-      // public properties
-      //////////////////////////////////////////////////////////////////////////
+      
+      
+      #region Public Properties
       public string Owner
       {
          get {return owner;}
-         set {owner = value;}
       }
 
       public ByteVector Identifier
@@ -66,14 +65,31 @@ namespace TagLib.Id3v2
          get {return identifier;}
          set {identifier = value;}
       }
-
-
-      //////////////////////////////////////////////////////////////////////////
-      // protected methods
-      //////////////////////////////////////////////////////////////////////////
-      protected override void ParseFields (ByteVector data)
+      #endregion
+      
+      
+      
+      #region Public Static Methods
+      public static UniqueFileIdentifierFrame Get (Tag tag, string owner, bool create)
       {
-         ByteVectorList fields = ByteVectorList.Split(data, (byte) 0);
+         foreach (Frame f in tag.GetFrames (FrameType.UFID))
+            if (f is UniqueFileIdentifierFrame && (f as UniqueFileIdentifierFrame).Owner == owner)
+               return f as UniqueFileIdentifierFrame;
+         
+         if (!create) return null;
+         
+         UniqueFileIdentifierFrame frame = new UniqueFileIdentifierFrame (owner, null);
+         tag.AddFrame (frame);
+         return frame;
+      }
+      #endregion
+      
+      
+      
+      #region Protected Methods
+      protected override void ParseFields (ByteVector data, byte version)
+      {
+         ByteVectorCollection fields = ByteVectorCollection.Split(data, (byte) 0);
 
          if (fields.Count != 2)
             return;
@@ -82,22 +98,16 @@ namespace TagLib.Id3v2
          identifier = fields [1];
       }
       
-      protected override ByteVector RenderFields ()
+      protected override ByteVector RenderFields (byte version)
       {
          ByteVector data = new ByteVector ();
-
+         
          data.Add (ByteVector.FromString (owner, StringType.Latin1));
-         data.Add (TextDelimiter (StringType.Latin1));
+         data.Add (ByteVector.TextDelimiter (StringType.Latin1));
          data.Add (identifier);
-
+         
          return data;
       }
-      
-      protected internal UniqueFileIdentifierFrame (ByteVector data, int offset, FrameHeader h) : base (h)
-      {
-         owner = null;
-         identifier = null;
-         ParseFields (FieldData (data, offset));
-      }
+      #endregion
    }
 }
