@@ -383,7 +383,7 @@ namespace Banshee.Dap
             Queue remove_queue = new Queue();
             
             foreach(TrackInfo track in Tracks) {
-                if(track == null || track.Uri == null) {
+                if(track == null || track.Uri == null || !track.Uri.IsFile) {
                     continue;
                 }
             
@@ -524,6 +524,10 @@ namespace Banshee.Dap
                 supported_extensions = extensions.ToArray(typeof(string)) as string [];
                 return supported_extensions;
             }
+
+            set {
+                supported_extensions = value;
+            }
         }
         
         protected string PipelineCodecFilter {
@@ -677,6 +681,10 @@ namespace Banshee.Dap
                 
                 return supported_playback_mime_types;
             }
+
+            protected set {
+                supported_playback_mime_types = value;
+            }
         }
         
         private string id;
