@@ -48,6 +48,8 @@ using Banshee.Sources;
 using Banshee.Gui;
 using Banshee.Gui.DragDrop;
 using Banshee.Configuration.Schema;
+using Banshee.Playlists;
+using Banshee.Playlists.Formats;
 
 namespace Banshee
 {
@@ -417,6 +419,7 @@ namespace Banshee
             playlistModel.Updated += OnPlaylistUpdated;
             playlistModel.Stopped += OnPlaylistStopped;
             playlistView.KeyPressEvent += OnPlaylistViewKeyPressEvent;
+            playlistView.PopupMenu += OnPlaylistViewPopupMenu;
             playlistView.ButtonPressEvent += OnPlaylistViewButtonPressEvent;
             playlistView.MotionNotifyEvent += OnPlaylistViewMotionNotifyEvent;
             playlistView.ButtonReleaseEvent += OnPlaylistViewButtonReleaseEvent;
@@ -443,18 +446,19 @@ namespace Banshee
             Window.DeleteEvent += OnWindowPlayerDeleteEvent;
             WindowPlayer.WindowStateEvent += OnWindowStateEvent;
             
-            // Search Entry
-            ArrayList fields = new ArrayList();
-            fields.Add(Catalog.GetString("All"));
-            fields.Add("-");
-            fields.Add(Catalog.GetString("Song Name"));
-            fields.Add(Catalog.GetString("Artist Name"));
-            fields.Add(Catalog.GetString("Album Title"));
-            fields.Add(Catalog.GetString("Genre"));
-            fields.Add(Catalog.GetString("Year"));
+            searchEntry = new SearchEntry();
+            searchEntry.Changed += OnSimpleSearch;
+            searchEntry.FilterChanged += OnSimpleSearchFilterChange;
+            searchEntry.SetSizeRequest(200, -1);
+            searchEntry.AddFilterOption((int)TrackFilterType.None, Catalog.GetString("All Columns"));
+            searchEntry.AddFilterSeparator();
+            searchEntry.AddFilterOption((int)TrackFilterType.SongName, Catalog.GetString("Song Name"));
+            searchEntry.AddFilterOption((int)TrackFilterType.ArtistName, Catalog.GetString("Artist Name"));
+            searchEntry.AddFilterOption((int)TrackFilterType.AlbumTitle, Catalog.GetString("Album Title"));
+            searchEntry.AddFilterOption((int)TrackFilterType.Genre, Catalog.GetString("Genre"));
+            searchEntry.AddFilterOption((int)TrackFilterType.Year, Catalog.GetString("Year"));
             
-            searchEntry = new SearchEntry(fields);
-            searchEntry.EnterPress += delegate(object o, EventArgs args) {
+            searchEntry.Activated += delegate {
                 if(!SourceManager.ActiveSource.HandlesSearch) {
                     if(playlistView.Selection.CountSelectedRows() == 0 && playlistModel.Count() > 0) {
                         playlistView.Selection.SelectPath(new TreePath(new int [] { 0 }));
@@ -462,7 +466,6 @@ namespace Banshee
                     playlistView.HasFocus = true;
                 }
             };
-            searchEntry.Changed += OnSimpleSearch;
             searchEntry.Show();
             gxml["SearchLabel"].Sensitive = true;
             searchEntry.Sensitive = true;
@@ -727,7 +730,7 @@ namespace Banshee
             }
 
             if(focus_search && !searchEntry.HasFocus && !sourceView.EditingRow) {
-                searchEntry.Focus();
+                searchEntry.HasFocus = true;
                 handled = true;
             }
             
@@ -744,6 +747,13 @@ namespace Banshee
                     break;
             }
         }       
+
+        [GLib.ConnectBefore]
+        private void OnPlaylistViewPopupMenu(object o, PopupMenuArgs args)
+        {
+            PlaylistMenuPopupTimeout(Gtk.Global.CurrentEventTime);
+            args.RetVal = true;
+        }
               
         // ---- Playback Event Handlers ----
         
@@ -1143,9 +1153,9 @@ namespace Banshee
             searchEntry.Ready = false;
             searchEntry.CancelSearch();
             
-            if(source.FilterQuery != null && source.FilterField != null) {
+            if(source.FilterQuery != null && source.FilterType != TrackFilterType.None) {
                 searchEntry.Query = source.FilterQuery;
-                searchEntry.Field = source.FilterField;
+                searchEntry.ActivateFilter((int)source.FilterType);
             }
             
             if(source is DapSource) {
@@ -1326,18 +1336,18 @@ namespace Banshee
             }
             
             string query = searchEntry.Query;
-            string field = searchEntry.Field;
+            TrackFilterType filter_type = (TrackFilterType)searchEntry.ActiveFilterID;
             string [] matches;
             
-            if(field == Catalog.GetString("Artist Name")) {
+            if((filter_type & TrackFilterType.ArtistName) == TrackFilterType.ArtistName) {
                 matches = new string [] { ti.Artist };
-            } else if(field == Catalog.GetString("Song Name")) {
+            } else if((filter_type & TrackFilterType.SongName) == TrackFilterType.SongName) {
                 matches = new string [] { ti.Title };
-            } else if(field == Catalog.GetString("Album Title")) {
+            } else if((filter_type & TrackFilterType.AlbumTitle) == TrackFilterType.AlbumTitle) {
                 matches = new string [] { ti.Album };
-            } else if(field == Catalog.GetString("Genre")) {
+            } else if((filter_type & TrackFilterType.Genre) == TrackFilterType.Genre) {
                 matches = new string [] { ti.Genre };
-            } else if(field == Catalog.GetString("Year")) {
+            } else if((filter_type & TrackFilterType.Year) == TrackFilterType.Year) {
                 matches = new string [] { ti.Year.ToString() };
             } else {
                 matches = new string [] {
@@ -1397,19 +1407,22 @@ namespace Banshee
             return true;
         }
         
+        private void OnSimpleSearchFilterChange(object o, EventArgs args)
+        {
+            searchEntry.EmptyMessage = String.Format(Catalog.GetString("Filter on {0}"), 
+                searchEntry.GetLabelForFilterID(searchEntry.ActiveFilterID));
+        }
+        
         private void OnSimpleSearch(object o, EventArgs args)
         {
-            SourceManager.ActiveSource.FilterField = searchEntry.Field;
+            SourceManager.ActiveSource.FilterType = (TrackFilterType)searchEntry.ActiveFilterID;
             SourceManager.ActiveSource.FilterQuery = searchEntry.Query;
         
-            if(SourceManager.ActiveSource.HandlesSearch) {
+            if(SourceManager.ActiveSource.HandlesSearch || suspendSearch || 
+                (searchEntry.IsQueryAvailable && searchEntry.Query.Length < 3)) {
                 return;
             }
 
-            if (suspendSearch) {
-                return;
-            }
-            
             playlistModel.ClearModel();
             
             if(!searchEntry.IsQueryAvailable) {
@@ -1574,25 +1587,34 @@ namespace Banshee
 
         private Menu song_popup_menu = null;
         private MenuItem add_to_playlist_menu_item = null;
-        private MenuItem rating_menu_item = null;
+        private RatingMenuItem rating_menu_item = null;
         
         private bool PlaylistMenuPopupTimeout(uint time)
         {
             if(song_popup_menu == null) {
                 song_popup_menu = Globals.ActionManager.GetWidget("/SongViewPopup") as Menu;
-                add_to_playlist_menu_item = Globals.ActionManager.GetWidget(
-                    "/SongViewPopup/AddToPlaylist") as MenuItem;
-                rating_menu_item = Globals.ActionManager.GetWidget("/SongViewPopup/Rating") as MenuItem;
+                add_to_playlist_menu_item = Globals.ActionManager.GetWidget("/SongViewPopup/AddToPlaylist") as MenuItem;
+                
+                for(int i = 0; i < song_popup_menu.Children.Length; i++) {
+                    if(song_popup_menu.Children[i] == add_to_playlist_menu_item) {
+                        rating_menu_item = new RatingMenuItem();
+                        rating_menu_item.Activated += OnItemRatingActivated;
+                        rating_menu_item.Visible = false;
+                        song_popup_menu.Insert(rating_menu_item, i + 1);
+                        break;
+                    }
+                }
             }
           
-            bool sensitive = playlistView.Selection.CountSelectedRows() > 0;
+            int selected_row_count = playlistView.Selection.CountSelectedRows();
+            bool sensitive = selected_row_count > 0;
 
             if(sensitive && (SourceManager.ActiveSource is LibrarySource ||
                     SourceManager.ActiveSource is PlaylistSource ||
                     SourceManager.ActiveSource is Banshee.SmartPlaylist.SmartPlaylistSource)) {
 
                 Globals.ActionManager["AddToPlaylistAction"].Visible = true;
-                Globals.ActionManager["RatingAction"].Visible = true;
+                rating_menu_item.Visible = true;
             
                 Menu plMenu = new Menu();
                 playlistMenuMap.Clear();
@@ -1605,50 +1627,34 @@ namespace Banshee
                 if(PlaylistSource.PlaylistCount > 0) {
                     plMenu.Append(new SeparatorMenuItem());
                     
-                    foreach(PlaylistSource playlist in PlaylistSource.Playlists) {
-                        ImageMenuItem item = new ImageMenuItem(playlist.Name);
-                        item.Image = new Gtk.Image(Pixbuf.LoadFromResource("source-playlist.png"));
-                        item.Activated += OnItemAddToPlaylistActivated;
-                        playlistMenuMap[item] = playlist;
-                        plMenu.Append(item);
+                    foreach(ChildSource playlist in LibrarySource.Instance.Children) {               
+                        if (playlist is PlaylistSource) {
+                            ImageMenuItem item = new ImageMenuItem(playlist.Name);
+                            item.Image = new Gtk.Image(Pixbuf.LoadFromResource("source-playlist-16.png"));
+                            item.Activated += OnItemAddToPlaylistActivated;
+                            playlistMenuMap[item] = playlist;
+                            plMenu.Append(item);
+                        }
                     }
                 }
                 
-                Menu ratingMenu = new Menu();
-                
-                MenuItem clearItem = new MenuItem(Catalog.GetString("Clear"));
-                clearItem.Name = "0";
-                clearItem.Activated += OnItemRatingActivated;
-                
-                ratingMenu.Append(clearItem);
-                ratingMenu.Append(new SeparatorMenuItem());
-                
-                for(int i = 0; i < 5; i++) {
-                    MenuItem item = new MenuItem();
-                    HBox box = new HBox();
-                    box.Spacing = 3;
-                    
-                    for(int j = 0; j < i + 1; j++) {
-                        box.PackStart(new Gtk.Image(Banshee.Gui.CellRendererRating.RatedPixbuf), false, false, 0);
-                    }
-                    
-                    item.Add(box);
-                    item.Name = String.Format("{0}", i + 1);
-                    item.Activated += OnItemRatingActivated;
-                    ratingMenu.Append(item);
-                }
-                
                 add_to_playlist_menu_item.Submenu = plMenu;
-                rating_menu_item.Submenu = ratingMenu;
-                
                 plMenu.ShowAll();
-                ratingMenu.ShowAll();
+                
+                // FIXME: The selection/popup order is fux0red
+                // if(selected_row_count == 1) {
+                //    rating_menu_item.Reset((int)playlistView.SelectedTrackInfo.Rating);
+                // } else {
+                //    rating_menu_item.Reset(0);
+                // }
+                
+                rating_menu_item.Reset(0);
             } else {
                 Globals.ActionManager["AddToPlaylistAction"].Visible = false;
-                Globals.ActionManager["RatingAction"].Visible = false;
+                rating_menu_item.Visible = false;
             }
         
-            song_popup_menu.ShowAll();
+            song_popup_menu.Show();
             song_popup_menu.Popup(null, null, null, 0, time);
             
             return false;
@@ -1656,9 +1662,12 @@ namespace Banshee
 
         private void OnItemRatingActivated(object o, EventArgs args)
         {
-            uint rating = Convert.ToUInt32((o as Widget).Name);
-            foreach(TreePath path in playlistView.Selection.GetSelectedRows())
+            uint rating = (uint)(o as RatingMenuItem).Value;
+            
+            foreach(TreePath path in playlistView.Selection.GetSelectedRows()) {
                 playlistModel.PathTrackInfo(path).Rating = rating;
+            }
+            
             playlistView.QueueDraw();
         }
 
@@ -1996,6 +2005,98 @@ namespace Banshee
             PromptForImport(false);
         }
         
+        private void OnImportPlaylistAction(object o, EventArgs args)
+        {
+            // Prompt user for location of the playlist.
+            Banshee.Gui.Dialogs.FileChooserDialog chooser = new Banshee.Gui.Dialogs.FileChooserDialog(
+                Catalog.GetString("Import Playlist"),
+                WindowPlayer,
+                FileChooserAction.Open
+            );
+                         
+            chooser.DefaultResponse = ResponseType.Ok;
+            chooser.SelectMultiple = false;
+
+            chooser.AddButton(Stock.Cancel, ResponseType.Cancel);
+            chooser.AddButton(Catalog.GetString("Import"), ResponseType.Ok);
+            
+            string playlist_uri = null;
+            int response = chooser.Run();            
+
+            if(response == (int) ResponseType.Ok) {            	
+                playlist_uri = SafeUri.UriToFilename(chooser.Uri);              
+                chooser.Destroy(); 
+            } else {
+                // User cancelled import.
+                chooser.Destroy();                 
+                return;
+            } 
+
+            // Read the contents of the playlist.
+            string[] uris = null;
+            try {            	
+                uris = PlaylistFileUtil.ImportPlaylist(playlist_uri);            	
+            } catch (Exception e) {
+                HigMessageDialog md = new HigMessageDialog(WindowPlayer, 
+                    DialogFlags.DestroyWithParent, 
+                    MessageType.Error,  
+                    ButtonsType.Ok,
+                    Catalog.GetString("Unable to Import Playlist"),
+                    e.Message);
+
+                md.Run();
+                md.Destroy();
+                return;
+            }
+
+            // Import the tracks specified in the playlist.
+            if (uris != null) {
+                ImportPlaylistWorker worker = new ImportPlaylistWorker(uris);
+                Thread t = new Thread(new ThreadStart(worker.Import));		        
+                t.Start();
+            } else {
+                HigMessageDialog md = new HigMessageDialog(WindowPlayer, 
+                    DialogFlags.DestroyWithParent, 
+                    MessageType.Error,  
+                    ButtonsType.Ok,
+                    Catalog.GetString("Unable to Import Playlist"),
+                    Catalog.GetString("Banshee was unable to find any valid tracks to import.  " +
+                        "Please check the playlist and try again."));
+
+                md.Run();
+                md.Destroy();
+                return;
+            }
+        }
+        
+        private void OnExportPlaylistAction(object o, EventArgs args)
+        {   
+            Source source = sourceView.HighlightedSource;
+            if (!PlaylistFileUtil.IsSourceExportSupported(source)) {
+                return;
+            }
+
+            PlaylistExportDialog chooser = new PlaylistExportDialog(source.Name, WindowPlayer);
+
+            string uri = null;
+            PlaylistFile playlist = null;
+            int response = chooser.Run();            
+            if(response == (int) ResponseType.Ok) {            	
+                uri = SafeUri.UriToFilename(chooser.Uri);
+
+                // Get the format that the user selected.
+                playlist = chooser.GetExportFormat();
+            }             
+            chooser.Destroy(); 
+
+            if (uri == null) {
+                // User cancelled export.
+                return;
+            }
+
+            playlist.Export(uri, source);            
+        } 
+        
         private void OnOpenLocationAction(object o, EventArgs args)
         {
             Banshee.Gui.OpenLocationDialog dialog = new Banshee.Gui.OpenLocationDialog();
@@ -2376,15 +2477,15 @@ namespace Banshee
 
             switch(criteria) {
                 case SearchTrackCriteria.Album:
-                    searchEntry.Field = Catalog.GetString("Album Title");
+                    searchEntry.ActivateFilter((int)TrackFilterType.AlbumTitle);
                     searchEntry.Query = track.Album;
                     break;
                 case SearchTrackCriteria.Artist:
-                    searchEntry.Field = Catalog.GetString("Artist Name");
+                    searchEntry.ActivateFilter((int)TrackFilterType.ArtistName);
                     searchEntry.Query = track.Artist;
                     break;
                 case SearchTrackCriteria.Genre:
-                    searchEntry.Field = Catalog.GetString("Genre");
+                    searchEntry.ActivateFilter((int)TrackFilterType.Genre);
                     searchEntry.Query = track.Genre;
                     break;
             }
